# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class V1CloudSpaceApp(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'author_logo': 'str',
        'authors': 'list[str]',
        'categories': 'list[str]',
        'created_at': 'datetime',
        'description': 'str',
        'entrypoint_file': 'str',
        'git_url': 'str',
        'id': 'str',
        'license': 'str',
        'name': 'str',
        'owner_id': 'str',
        'owner_type': 'V1CloudSpaceAppOwnerType',
        'short_description': 'str',
        'source_url': 'str',
        'tags': 'list[str]',
        'thumbnail': 'str',
        'updated_at': 'datetime',
        'version': 'str',
        'visible_to_org_ids': 'list[str]',
        'visible_to_project_ids': 'list[str]',
        'visible_to_user_ids': 'list[str]'
    }

    attribute_map = {
        'author_logo': 'authorLogo',
        'authors': 'authors',
        'categories': 'categories',
        'created_at': 'createdAt',
        'description': 'description',
        'entrypoint_file': 'entrypointFile',
        'git_url': 'gitUrl',
        'id': 'id',
        'license': 'license',
        'name': 'name',
        'owner_id': 'ownerId',
        'owner_type': 'ownerType',
        'short_description': 'shortDescription',
        'source_url': 'sourceUrl',
        'tags': 'tags',
        'thumbnail': 'thumbnail',
        'updated_at': 'updatedAt',
        'version': 'version',
        'visible_to_org_ids': 'visibleToOrgIds',
        'visible_to_project_ids': 'visibleToProjectIds',
        'visible_to_user_ids': 'visibleToUserIds'
    }

    def __init__(self, author_logo: 'str' =None, authors: 'list[str]' =None, categories: 'list[str]' =None, created_at: 'datetime' =None, description: 'str' =None, entrypoint_file: 'str' =None, git_url: 'str' =None, id: 'str' =None, license: 'str' =None, name: 'str' =None, owner_id: 'str' =None, owner_type: 'V1CloudSpaceAppOwnerType' =None, short_description: 'str' =None, source_url: 'str' =None, tags: 'list[str]' =None, thumbnail: 'str' =None, updated_at: 'datetime' =None, version: 'str' =None, visible_to_org_ids: 'list[str]' =None, visible_to_project_ids: 'list[str]' =None, visible_to_user_ids: 'list[str]' =None):  # noqa: E501
        """V1CloudSpaceApp - a model defined in Swagger"""  # noqa: E501
        self._author_logo = None
        self._authors = None
        self._categories = None
        self._created_at = None
        self._description = None
        self._entrypoint_file = None
        self._git_url = None
        self._id = None
        self._license = None
        self._name = None
        self._owner_id = None
        self._owner_type = None
        self._short_description = None
        self._source_url = None
        self._tags = None
        self._thumbnail = None
        self._updated_at = None
        self._version = None
        self._visible_to_org_ids = None
        self._visible_to_project_ids = None
        self._visible_to_user_ids = None
        self.discriminator = None
        if author_logo is not None:
            self.author_logo = author_logo
        if authors is not None:
            self.authors = authors
        if categories is not None:
            self.categories = categories
        if created_at is not None:
            self.created_at = created_at
        if description is not None:
            self.description = description
        if entrypoint_file is not None:
            self.entrypoint_file = entrypoint_file
        if git_url is not None:
            self.git_url = git_url
        if id is not None:
            self.id = id
        if license is not None:
            self.license = license
        if name is not None:
            self.name = name
        if owner_id is not None:
            self.owner_id = owner_id
        if owner_type is not None:
            self.owner_type = owner_type
        if short_description is not None:
            self.short_description = short_description
        if source_url is not None:
            self.source_url = source_url
        if tags is not None:
            self.tags = tags
        if thumbnail is not None:
            self.thumbnail = thumbnail
        if updated_at is not None:
            self.updated_at = updated_at
        if version is not None:
            self.version = version
        if visible_to_org_ids is not None:
            self.visible_to_org_ids = visible_to_org_ids
        if visible_to_project_ids is not None:
            self.visible_to_project_ids = visible_to_project_ids
        if visible_to_user_ids is not None:
            self.visible_to_user_ids = visible_to_user_ids

    @property
    def author_logo(self) -> 'str':
        """Gets the author_logo of this V1CloudSpaceApp.  # noqa: E501


        :return: The author_logo of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._author_logo

    @author_logo.setter
    def author_logo(self, author_logo: 'str'):
        """Sets the author_logo of this V1CloudSpaceApp.


        :param author_logo: The author_logo of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._author_logo = author_logo

    @property
    def authors(self) -> 'list[str]':
        """Gets the authors of this V1CloudSpaceApp.  # noqa: E501


        :return: The authors of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._authors

    @authors.setter
    def authors(self, authors: 'list[str]'):
        """Sets the authors of this V1CloudSpaceApp.


        :param authors: The authors of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._authors = authors

    @property
    def categories(self) -> 'list[str]':
        """Gets the categories of this V1CloudSpaceApp.  # noqa: E501


        :return: The categories of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._categories

    @categories.setter
    def categories(self, categories: 'list[str]'):
        """Sets the categories of this V1CloudSpaceApp.


        :param categories: The categories of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._categories = categories

    @property
    def created_at(self) -> 'datetime':
        """Gets the created_at of this V1CloudSpaceApp.  # noqa: E501


        :return: The created_at of this V1CloudSpaceApp.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at: 'datetime'):
        """Sets the created_at of this V1CloudSpaceApp.


        :param created_at: The created_at of this V1CloudSpaceApp.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def description(self) -> 'str':
        """Gets the description of this V1CloudSpaceApp.  # noqa: E501


        :return: The description of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description: 'str'):
        """Sets the description of this V1CloudSpaceApp.


        :param description: The description of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def entrypoint_file(self) -> 'str':
        """Gets the entrypoint_file of this V1CloudSpaceApp.  # noqa: E501


        :return: The entrypoint_file of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._entrypoint_file

    @entrypoint_file.setter
    def entrypoint_file(self, entrypoint_file: 'str'):
        """Sets the entrypoint_file of this V1CloudSpaceApp.


        :param entrypoint_file: The entrypoint_file of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._entrypoint_file = entrypoint_file

    @property
    def git_url(self) -> 'str':
        """Gets the git_url of this V1CloudSpaceApp.  # noqa: E501


        :return: The git_url of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._git_url

    @git_url.setter
    def git_url(self, git_url: 'str'):
        """Sets the git_url of this V1CloudSpaceApp.


        :param git_url: The git_url of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._git_url = git_url

    @property
    def id(self) -> 'str':
        """Gets the id of this V1CloudSpaceApp.  # noqa: E501


        :return: The id of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id: 'str'):
        """Sets the id of this V1CloudSpaceApp.


        :param id: The id of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def license(self) -> 'str':
        """Gets the license of this V1CloudSpaceApp.  # noqa: E501


        :return: The license of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._license

    @license.setter
    def license(self, license: 'str'):
        """Sets the license of this V1CloudSpaceApp.


        :param license: The license of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._license = license

    @property
    def name(self) -> 'str':
        """Gets the name of this V1CloudSpaceApp.  # noqa: E501


        :return: The name of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: 'str'):
        """Sets the name of this V1CloudSpaceApp.


        :param name: The name of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def owner_id(self) -> 'str':
        """Gets the owner_id of this V1CloudSpaceApp.  # noqa: E501


        :return: The owner_id of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id: 'str'):
        """Sets the owner_id of this V1CloudSpaceApp.


        :param owner_id: The owner_id of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._owner_id = owner_id

    @property
    def owner_type(self) -> 'V1CloudSpaceAppOwnerType':
        """Gets the owner_type of this V1CloudSpaceApp.  # noqa: E501


        :return: The owner_type of this V1CloudSpaceApp.  # noqa: E501
        :rtype: V1CloudSpaceAppOwnerType
        """
        return self._owner_type

    @owner_type.setter
    def owner_type(self, owner_type: 'V1CloudSpaceAppOwnerType'):
        """Sets the owner_type of this V1CloudSpaceApp.


        :param owner_type: The owner_type of this V1CloudSpaceApp.  # noqa: E501
        :type: V1CloudSpaceAppOwnerType
        """

        self._owner_type = owner_type

    @property
    def short_description(self) -> 'str':
        """Gets the short_description of this V1CloudSpaceApp.  # noqa: E501


        :return: The short_description of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._short_description

    @short_description.setter
    def short_description(self, short_description: 'str'):
        """Sets the short_description of this V1CloudSpaceApp.


        :param short_description: The short_description of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._short_description = short_description

    @property
    def source_url(self) -> 'str':
        """Gets the source_url of this V1CloudSpaceApp.  # noqa: E501


        :return: The source_url of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._source_url

    @source_url.setter
    def source_url(self, source_url: 'str'):
        """Sets the source_url of this V1CloudSpaceApp.


        :param source_url: The source_url of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._source_url = source_url

    @property
    def tags(self) -> 'list[str]':
        """Gets the tags of this V1CloudSpaceApp.  # noqa: E501


        :return: The tags of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags: 'list[str]'):
        """Sets the tags of this V1CloudSpaceApp.


        :param tags: The tags of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._tags = tags

    @property
    def thumbnail(self) -> 'str':
        """Gets the thumbnail of this V1CloudSpaceApp.  # noqa: E501


        :return: The thumbnail of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._thumbnail

    @thumbnail.setter
    def thumbnail(self, thumbnail: 'str'):
        """Sets the thumbnail of this V1CloudSpaceApp.


        :param thumbnail: The thumbnail of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._thumbnail = thumbnail

    @property
    def updated_at(self) -> 'datetime':
        """Gets the updated_at of this V1CloudSpaceApp.  # noqa: E501


        :return: The updated_at of this V1CloudSpaceApp.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at: 'datetime'):
        """Sets the updated_at of this V1CloudSpaceApp.


        :param updated_at: The updated_at of this V1CloudSpaceApp.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def version(self) -> 'str':
        """Gets the version of this V1CloudSpaceApp.  # noqa: E501


        :return: The version of this V1CloudSpaceApp.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version: 'str'):
        """Sets the version of this V1CloudSpaceApp.


        :param version: The version of this V1CloudSpaceApp.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def visible_to_org_ids(self) -> 'list[str]':
        """Gets the visible_to_org_ids of this V1CloudSpaceApp.  # noqa: E501


        :return: The visible_to_org_ids of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._visible_to_org_ids

    @visible_to_org_ids.setter
    def visible_to_org_ids(self, visible_to_org_ids: 'list[str]'):
        """Sets the visible_to_org_ids of this V1CloudSpaceApp.


        :param visible_to_org_ids: The visible_to_org_ids of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._visible_to_org_ids = visible_to_org_ids

    @property
    def visible_to_project_ids(self) -> 'list[str]':
        """Gets the visible_to_project_ids of this V1CloudSpaceApp.  # noqa: E501


        :return: The visible_to_project_ids of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._visible_to_project_ids

    @visible_to_project_ids.setter
    def visible_to_project_ids(self, visible_to_project_ids: 'list[str]'):
        """Sets the visible_to_project_ids of this V1CloudSpaceApp.


        :param visible_to_project_ids: The visible_to_project_ids of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._visible_to_project_ids = visible_to_project_ids

    @property
    def visible_to_user_ids(self) -> 'list[str]':
        """Gets the visible_to_user_ids of this V1CloudSpaceApp.  # noqa: E501


        :return: The visible_to_user_ids of this V1CloudSpaceApp.  # noqa: E501
        :rtype: list[str]
        """
        return self._visible_to_user_ids

    @visible_to_user_ids.setter
    def visible_to_user_ids(self, visible_to_user_ids: 'list[str]'):
        """Sets the visible_to_user_ids of this V1CloudSpaceApp.


        :param visible_to_user_ids: The visible_to_user_ids of this V1CloudSpaceApp.  # noqa: E501
        :type: list[str]
        """

        self._visible_to_user_ids = visible_to_user_ids

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1CloudSpaceApp, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1CloudSpaceApp') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1CloudSpaceApp):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'V1CloudSpaceApp') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class V1GetCloudSpaceInstanceStatusResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'collab_session': 'V1CollabSession',
        'collab_status': 'V1CloudSpaceInstanceCollabStatus',
        'in_use': 'Externalv1CloudSpaceInstanceStatus',
        'requested': 'Externalv1CloudSpaceInstanceStatus'
    }

    attribute_map = {
        'collab_session': 'collabSession',
        'collab_status': 'collabStatus',
        'in_use': 'inUse',
        'requested': 'requested'
    }

    def __init__(self, collab_session: 'V1CollabSession' =None, collab_status: 'V1CloudSpaceInstanceCollabStatus' =None, in_use: 'Externalv1CloudSpaceInstanceStatus' =None, requested: 'Externalv1CloudSpaceInstanceStatus' =None):  # noqa: E501
        """V1GetCloudSpaceInstanceStatusResponse - a model defined in Swagger"""  # noqa: E501
        self._collab_session = None
        self._collab_status = None
        self._in_use = None
        self._requested = None
        self.discriminator = None
        if collab_session is not None:
            self.collab_session = collab_session
        if collab_status is not None:
            self.collab_status = collab_status
        if in_use is not None:
            self.in_use = in_use
        if requested is not None:
            self.requested = requested

    @property
    def collab_session(self) -> 'V1CollabSession':
        """Gets the collab_session of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501


        :return: The collab_session of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :rtype: V1CollabSession
        """
        return self._collab_session

    @collab_session.setter
    def collab_session(self, collab_session: 'V1CollabSession'):
        """Sets the collab_session of this V1GetCloudSpaceInstanceStatusResponse.


        :param collab_session: The collab_session of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :type: V1CollabSession
        """

        self._collab_session = collab_session

    @property
    def collab_status(self) -> 'V1CloudSpaceInstanceCollabStatus':
        """Gets the collab_status of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501


        :return: The collab_status of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :rtype: V1CloudSpaceInstanceCollabStatus
        """
        return self._collab_status

    @collab_status.setter
    def collab_status(self, collab_status: 'V1CloudSpaceInstanceCollabStatus'):
        """Sets the collab_status of this V1GetCloudSpaceInstanceStatusResponse.


        :param collab_status: The collab_status of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :type: V1CloudSpaceInstanceCollabStatus
        """

        self._collab_status = collab_status

    @property
    def in_use(self) -> 'Externalv1CloudSpaceInstanceStatus':
        """Gets the in_use of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501


        :return: The in_use of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :rtype: Externalv1CloudSpaceInstanceStatus
        """
        return self._in_use

    @in_use.setter
    def in_use(self, in_use: 'Externalv1CloudSpaceInstanceStatus'):
        """Sets the in_use of this V1GetCloudSpaceInstanceStatusResponse.


        :param in_use: The in_use of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :type: Externalv1CloudSpaceInstanceStatus
        """

        self._in_use = in_use

    @property
    def requested(self) -> 'Externalv1CloudSpaceInstanceStatus':
        """Gets the requested of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501


        :return: The requested of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :rtype: Externalv1CloudSpaceInstanceStatus
        """
        return self._requested

    @requested.setter
    def requested(self, requested: 'Externalv1CloudSpaceInstanceStatus'):
        """Sets the requested of this V1GetCloudSpaceInstanceStatusResponse.


        :param requested: The requested of this V1GetCloudSpaceInstanceStatusResponse.  # noqa: E501
        :type: Externalv1CloudSpaceInstanceStatus
        """

        self._requested = requested

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1GetCloudSpaceInstanceStatusResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1GetCloudSpaceInstanceStatusResponse') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1GetCloudSpaceInstanceStatusResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'V1GetCloudSpaceInstanceStatusResponse') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class V1ProjectMembership(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'balance': 'float',
        'created_at': 'datetime',
        'datastore_count': 'str',
        'description': 'str',
        'display_name': 'str',
        'free_credits_enabled': 'bool',
        'job_count': 'str',
        'membership_count': 'str',
        'name': 'str',
        'owner_id': 'str',
        'owner_type': 'V1OwnerType',
        'project_id': 'str',
        'quotas': 'V1Quotas',
        'roles': 'list[V1Role]',
        'updated_at': 'datetime',
        'user_id': 'str'
    }

    attribute_map = {
        'balance': 'balance',
        'created_at': 'createdAt',
        'datastore_count': 'datastoreCount',
        'description': 'description',
        'display_name': 'displayName',
        'free_credits_enabled': 'freeCreditsEnabled',
        'job_count': 'jobCount',
        'membership_count': 'membershipCount',
        'name': 'name',
        'owner_id': 'ownerId',
        'owner_type': 'ownerType',
        'project_id': 'projectId',
        'quotas': 'quotas',
        'roles': 'roles',
        'updated_at': 'updatedAt',
        'user_id': 'userId'
    }

    def __init__(self, balance: 'float' =None, created_at: 'datetime' =None, datastore_count: 'str' =None, description: 'str' =None, display_name: 'str' =None, free_credits_enabled: 'bool' =None, job_count: 'str' =None, membership_count: 'str' =None, name: 'str' =None, owner_id: 'str' =None, owner_type: 'V1OwnerType' =None, project_id: 'str' =None, quotas: 'V1Quotas' =None, roles: 'list[V1Role]' =None, updated_at: 'datetime' =None, user_id: 'str' =None):  # noqa: E501
        """V1ProjectMembership - a model defined in Swagger"""  # noqa: E501
        self._balance = None
        self._created_at = None
        self._datastore_count = None
        self._description = None
        self._display_name = None
        self._free_credits_enabled = None
        self._job_count = None
        self._membership_count = None
        self._name = None
        self._owner_id = None
        self._owner_type = None
        self._project_id = None
        self._quotas = None
        self._roles = None
        self._updated_at = None
        self._user_id = None
        self.discriminator = None
        if balance is not None:
            self.balance = balance
        if created_at is not None:
            self.created_at = created_at
        if datastore_count is not None:
            self.datastore_count = datastore_count
        if description is not None:
            self.description = description
        if display_name is not None:
            self.display_name = display_name
        if free_credits_enabled is not None:
            self.free_credits_enabled = free_credits_enabled
        if job_count is not None:
            self.job_count = job_count
        if membership_count is not None:
            self.membership_count = membership_count
        if name is not None:
            self.name = name
        if owner_id is not None:
            self.owner_id = owner_id
        if owner_type is not None:
            self.owner_type = owner_type
        if project_id is not None:
            self.project_id = project_id
        if quotas is not None:
            self.quotas = quotas
        if roles is not None:
            self.roles = roles
        if updated_at is not None:
            self.updated_at = updated_at
        if user_id is not None:
            self.user_id = user_id

    @property
    def balance(self) -> 'float':
        """Gets the balance of this V1ProjectMembership.  # noqa: E501


        :return: The balance of this V1ProjectMembership.  # noqa: E501
        :rtype: float
        """
        return self._balance

    @balance.setter
    def balance(self, balance: 'float'):
        """Sets the balance of this V1ProjectMembership.


        :param balance: The balance of this V1ProjectMembership.  # noqa: E501
        :type: float
        """

        self._balance = balance

    @property
    def created_at(self) -> 'datetime':
        """Gets the created_at of this V1ProjectMembership.  # noqa: E501


        :return: The created_at of this V1ProjectMembership.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at: 'datetime'):
        """Sets the created_at of this V1ProjectMembership.


        :param created_at: The created_at of this V1ProjectMembership.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def datastore_count(self) -> 'str':
        """Gets the datastore_count of this V1ProjectMembership.  # noqa: E501


        :return: The datastore_count of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._datastore_count

    @datastore_count.setter
    def datastore_count(self, datastore_count: 'str'):
        """Sets the datastore_count of this V1ProjectMembership.


        :param datastore_count: The datastore_count of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._datastore_count = datastore_count

    @property
    def description(self) -> 'str':
        """Gets the description of this V1ProjectMembership.  # noqa: E501


        :return: The description of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description: 'str'):
        """Sets the description of this V1ProjectMembership.


        :param description: The description of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def display_name(self) -> 'str':
        """Gets the display_name of this V1ProjectMembership.  # noqa: E501


        :return: The display_name of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name: 'str'):
        """Sets the display_name of this V1ProjectMembership.


        :param display_name: The display_name of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def free_credits_enabled(self) -> 'bool':
        """Gets the free_credits_enabled of this V1ProjectMembership.  # noqa: E501


        :return: The free_credits_enabled of this V1ProjectMembership.  # noqa: E501
        :rtype: bool
        """
        return self._free_credits_enabled

    @free_credits_enabled.setter
    def free_credits_enabled(self, free_credits_enabled: 'bool'):
        """Sets the free_credits_enabled of this V1ProjectMembership.


        :param free_credits_enabled: The free_credits_enabled of this V1ProjectMembership.  # noqa: E501
        :type: bool
        """

        self._free_credits_enabled = free_credits_enabled

    @property
    def job_count(self) -> 'str':
        """Gets the job_count of this V1ProjectMembership.  # noqa: E501


        :return: The job_count of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._job_count

    @job_count.setter
    def job_count(self, job_count: 'str'):
        """Sets the job_count of this V1ProjectMembership.


        :param job_count: The job_count of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._job_count = job_count

    @property
    def membership_count(self) -> 'str':
        """Gets the membership_count of this V1ProjectMembership.  # noqa: E501


        :return: The membership_count of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._membership_count

    @membership_count.setter
    def membership_count(self, membership_count: 'str'):
        """Sets the membership_count of this V1ProjectMembership.


        :param membership_count: The membership_count of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._membership_count = membership_count

    @property
    def name(self) -> 'str':
        """Gets the name of this V1ProjectMembership.  # noqa: E501


        :return: The name of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: 'str'):
        """Sets the name of this V1ProjectMembership.


        :param name: The name of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def owner_id(self) -> 'str':
        """Gets the owner_id of this V1ProjectMembership.  # noqa: E501


        :return: The owner_id of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id: 'str'):
        """Sets the owner_id of this V1ProjectMembership.


        :param owner_id: The owner_id of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._owner_id = owner_id

    @property
    def owner_type(self) -> 'V1OwnerType':
        """Gets the owner_type of this V1ProjectMembership.  # noqa: E501


        :return: The owner_type of this V1ProjectMembership.  # noqa: E501
        :rtype: V1OwnerType
        """
        return self._owner_type

    @owner_type.setter
    def owner_type(self, owner_type: 'V1OwnerType'):
        """Sets the owner_type of this V1ProjectMembership.


        :param owner_type: The owner_type of this V1ProjectMembership.  # noqa: E501
        :type: V1OwnerType
        """

        self._owner_type = owner_type

    @property
    def project_id(self) -> 'str':
        """Gets the project_id of this V1ProjectMembership.  # noqa: E501


        :return: The project_id of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id: 'str'):
        """Sets the project_id of this V1ProjectMembership.


        :param project_id: The project_id of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def quotas(self) -> 'V1Quotas':
        """Gets the quotas of this V1ProjectMembership.  # noqa: E501


        :return: The quotas of this V1ProjectMembership.  # noqa: E501
        :rtype: V1Quotas
        """
        return self._quotas

    @quotas.setter
    def quotas(self, quotas: 'V1Quotas'):
        """Sets the quotas of this V1ProjectMembership.


        :param quotas: The quotas of this V1ProjectMembership.  # noqa: E501
        :type: V1Quotas
        """

        self._quotas = quotas

    @property
    def roles(self) -> 'list[V1Role]':
        """Gets the roles of this V1ProjectMembership.  # noqa: E501


        :return: The roles of this V1ProjectMembership.  # noqa: E501
        :rtype: list[V1Role]
        """
        return self._roles

    @roles.setter
    def roles(self, roles: 'list[V1Role]'):
        """Sets the roles of this V1ProjectMembership.


        :param roles: The roles of this V1ProjectMembership.  # noqa: E501
        :type: list[V1Role]
        """

        self._roles = roles

    @property
    def updated_at(self) -> 'datetime':
        """Gets the updated_at of this V1ProjectMembership.  # noqa: E501


        :return: The updated_at of this V1ProjectMembership.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at: 'datetime'):
        """Sets the updated_at of this V1ProjectMembership.


        :param updated_at: The updated_at of this V1ProjectMembership.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def user_id(self) -> 'str':
        """Gets the user_id of this V1ProjectMembership.  # noqa: E501


        :return: The user_id of this V1ProjectMembership.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id: 'str'):
        """Sets the user_id of this V1ProjectMembership.


        :param user_id: The user_id of this V1ProjectMembership.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1ProjectMembership, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'V1ProjectMembership') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ProjectMembership):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'V1ProjectMembership') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



import os
from typing import Optional, Union
from lightning_cloud.rest_client import LightningClient
from pathlib import Path
import sys
from time import sleep
from urllib import parse
from dataclasses import dataclass
import datetime
import re

# To avoid adding lightning_utilities as a dependency for now.
try:
    import boto3
    import botocore
    _BOTO3_AVAILABLE = True
except Exception:
    _BOTO3_AVAILABLE = False


try:
    from lightning_sdk import Studio, Machine
    _LIGHTNING_SDK_AVAILABLE = True
except:
    class Machine:
        pass
    _LIGHTNING_SDK_AVAILABLE = False


@dataclass
class Dir:
    """Holds a directory path and possibly its associated remote URL."""

    path: Optional[str] = None
    url: Optional[str] = None


def _resolve_dir(dir_path: Optional[Union[str, Dir]]) -> Dir:
    if isinstance(dir_path, Dir):
        return Dir(
            path=str(dir_path.path) if dir_path.path else None,
            url=str(dir_path.url) if dir_path.url else None
        )

    if dir_path is None:
        return Dir()

    if not isinstance(dir_path, str):
        raise ValueError(f"`dir_path` must be a `Dir` or a string, got: {dir_path}")

    if dir_path.startswith("s3://"):
        return Dir(path=None, url=dir_path)

    dir_path = _resolve_time_template(dir_path)

    dir_path_absolute = str(Path(dir_path).absolute().resolve())

    if dir_path_absolute.startswith("/teamspace/studios/this_studio"):
        return Dir(path=dir_path_absolute, url=None)

    if dir_path_absolute.startswith("/.project/cloudspaces") and len(dir_path_absolute.split("/")) > 3:
        return _resolve_studio(dir_path_absolute, None, dir_path_absolute.split("/")[3])

    if dir_path_absolute.startswith("/teamspace/studios") and len(dir_path_absolute.split("/")) > 3:
        return _resolve_studio(dir_path_absolute, dir_path_absolute.split("/")[3], None)

    if dir_path_absolute.startswith("/teamspace/s3_connections") and len(dir_path_absolute.split("/")) > 3:
        return _resolve_s3_connections(dir_path_absolute)

    if dir_path_absolute.startswith("/teamspace/datasets") and len(dir_path_absolute.split("/")) > 3:
        return _resolve_datasets(dir_path_absolute)

    return Dir(path=dir_path_absolute, url=None)


def _resolve_studio(dir_path: str, target_name: str, target_id: str) -> str:
    client = LightningClient(retry=False)

    # Get the ids from env variables
    cluster_id = os.getenv("LIGHTNING_CLUSTER_ID", None)
    project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)

    if cluster_id is None:
        raise RuntimeError("The `cluster_id` couldn't be found from the environement variables.")

    if project_id is None:
        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")

    clusters = client.cluster_service_list_project_clusters(project_id).clusters

    target_cloud_space = [
        cloudspace
        for cloudspace in client.cloud_space_service_list_cloud_spaces(
            project_id=project_id, cluster_id=cluster_id
        ).cloudspaces
        if cloudspace.name == target_name or cloudspace.id == target_id
    ]

    if not target_cloud_space:
        raise ValueError(f"We didn't find any matching Studio for the provided name `{target_name}`.")

    target_cluster = [cluster for cluster in clusters if cluster.id == target_cloud_space[0].cluster_id]

    if not target_cluster:
        raise ValueError(
            f"We didn't find a matching cluster associated with the id {target_cloud_space[0].cluster_id}."
        )

    bucket_name = target_cluster[0].spec.aws_v1.bucket_name

    return Dir(
        path=dir_path,
        url=os.path.join(
            f"s3://{bucket_name}/projects/{project_id}/cloudspaces/{target_cloud_space[0].id}/code/content",
            *dir_path.split("/")[4:],
        )
    )

def _resolve_s3_connections(dir_path: str) -> str:
    client = LightningClient(retry=False)

    # Get the ids from env variables
    project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
    if project_id is None:
        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")

    target_name = dir_path.split("/")[3]

    data_connections = client.data_connection_service_list_data_connections(project_id).data_connections

    data_connection = [dc for dc in data_connections if dc.name == target_name]

    if not data_connection:
        raise ValueError(f"We didn't find any matching data connection with the provided name `{target_name}`.")

    return Dir(
        path=dir_path,
        url=os.path.join(data_connection[0].aws.source, *dir_path.split("/")[4:])
    )

def _resolve_datasets(dir_path: str) -> str:
    client = LightningClient(retry=False)

    # Get the ids from env variables
    cluster_id = os.getenv("LIGHTNING_CLUSTER_ID", None)
    project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
    cloud_space_id = os.getenv("LIGHTNING_CLOUD_SPACE_ID", None)

    if cluster_id is None:
        raise RuntimeError("The `cluster_id` couldn't be found from the environement variables.")

    if project_id is None:
        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")

    if cloud_space_id is None:
        raise RuntimeError("The `cloud_space_id` couldn't be found from the environement variables.")

    clusters = client.cluster_service_list_project_clusters(project_id).clusters

    target_cloud_space = [
        cloudspace
        for cloudspace in client.cloud_space_service_list_cloud_spaces(
            project_id=project_id, cluster_id=cluster_id
        ).cloudspaces
        if cloudspace.id == cloud_space_id
    ]

    if not target_cloud_space:
        raise ValueError(f"We didn't find any matching Studio for the provided id `{cloud_space_id}`.")

    target_cluster = [cluster for cluster in clusters if cluster.id == target_cloud_space[0].cluster_id]

    if not target_cluster:
        raise ValueError(
            f"We didn't find a matching cluster associated with the id {target_cloud_space[0].cluster_id}."
        )

    return Dir(
        path=dir_path,
        url=os.path.join(
            f"s3://{target_cluster[0].spec.aws_v1.bucket_name}/projects/{project_id}/datasets/",
            *dir_path.split("/")[3:],
        )
    )


def _assert_dir_is_empty(output_dir: Dir, append: bool = False, overwrite: bool = False) -> None:
    if not isinstance(output_dir, Dir):
        raise ValueError("The provided output_dir isn't a Dir Object.")

    if output_dir.url is None:
        return

    obj = parse.urlparse(output_dir.url)

    if obj.scheme != "s3":
        raise ValueError(f'The provided folder should start with s3://. Found {output_dir.path}.')

    s3 = boto3.client("s3")

    objects = s3.list_objects_v2(
        Bucket=obj.netloc,
        Delimiter="/",
        Prefix=obj.path.lstrip("/").rstrip("/") + "/",
    )

    # We aren't alloweing to add more data
    # TODO: Add support for `append` and `overwrite`.
    if objects['KeyCount'] > 0:
        raise RuntimeError(
            f"The provided output_dir `{output_dir.path}` already contains data and datasets are meant to be immutable."
            " HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?")


def _assert_dir_has_index_file(output_dir: Dir) -> None:
    if not isinstance(output_dir, Dir):
        raise ValueError("The provided output_dir isn't a Dir Object.")

    if output_dir.url is None:
        return

    obj = parse.urlparse(output_dir.url)

    if obj.scheme != "s3":
        raise ValueError(f'The provided folder should start with s3://. Found {output_dir.path}.')

    s3 = boto3.client("s3")

    prefix = obj.path.lstrip("/").rstrip("/") + "/"

    objects = s3.list_objects_v2(
        Bucket=obj.netloc,
        Delimiter="/",
        Prefix=prefix,
    )

    # No files are found in this folder
    if objects['KeyCount'] == 0:
        return

    # Check the index file exists
    try:
        s3.head_object(Bucket=obj.netloc, Key=os.path.join(prefix, "index.json"))
        has_index_file = True
    except botocore.exceptions.ClientError:
        has_index_file = False

    if has_index_file:
        raise RuntimeError(
            f"The provided output_dir `{output_dir.path}` already contains an optimized immutable datasets."
            " HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?")
    else:
        bucket_name = obj.netloc
        s3 = boto3.resource('s3')
        for obj in s3.Bucket(bucket_name).objects.filter(Prefix=prefix):
            s3.Object(bucket_name, obj.key).delete()


def _get_lightning_cloud_url() -> str:
    # detect local development
    if os.getenv("VSCODE_PROXY_URI", "").startswith("http://localhost:9800"):
        return "http://localhost:9800"
    # DO NOT CHANGE!
    return os.getenv("LIGHTNING_CLOUD_URL", "https://lightning.ai")


def _resolve_time_template(path: str) -> str:
    match = re.search('^.*{%.*}$', path)
    if match is None:
        return path

    pattern = path.split("{")[1].split("}")[0]

    return path.replace("{" + pattern + "}", datetime.datetime.now().strftime(pattern))


def _execute(
    name: str,
    num_nodes: int,
    machine: Optional[Machine] = None,
    command: Optional[str] = None,
):
    """Remotely execute the current operator."""

    if not _LIGHTNING_SDK_AVAILABLE:
        raise ModuleNotFoundError("The `lightning_sdk` is required.")

    studio = Studio()
    job = studio._studio_api.create_data_prep_machine_job(
        command or f"cd {os.getcwd()} && python {' '.join(sys.argv)}",
        name=name,
        num_instances=num_nodes,
        studio_id=studio._studio.id,
        teamspace_id=studio._teamspace.id,
        cluster_id=studio._studio.cluster_id,
        cloud_compute=machine or studio._studio_api.get_machine(studio._studio.id, studio._teamspace.id),
    )

    has_printed = False

    while True:
        curr_job = studio._studio_api._client.lightningapp_instance_service_get_lightningapp_instance(
            project_id=studio._teamspace.id, id=job.id
        )
        if not has_printed:
            cloud_url = _get_lightning_cloud_url()
            job_url = f"{cloud_url}/{studio._user.username}/{studio._teamspace.name}"
            job_url += f"/studios/{studio.name}/app?app_id=data-prep&job_name={curr_job.name}"
            print(f"Find your job at {job_url}")
            has_printed = True

        if curr_job.status.phase == "LIGHTNINGAPP_INSTANCE_STATE_FAILED":
            raise RuntimeError(f"job {curr_job.name} failed!")

        if curr_job.status.phase == "LIGHTNINGAPP_INSTANCE_STATE_STOPPED":
            break

        sleep(1)


import time

import requests
from requests.adapters import HTTPAdapter
from rich.progress import BarColumn, Progress, TextColumn
from urllib3.util.retry import Retry


class FileUploader:
    """
    This class uploads a source file with presigned url to S3

    Attributes
    ----------
    source_file: str
        Source file to upload
    presigned_url: str
        Presigned urls dictionary, with key as part number and values as urls
    retries: int
        Amount of retries when requests encounter an error
    total_size: int
        Size of all files to upload
    name: str
        Name of this upload to display progress
    """
    workers: int = 8
    retries: int = 10000
    disconnect_retry_wait_seconds: int = 5

    progress = Progress(
        TextColumn("[bold blue]{task.description}", justify="left"),
        BarColumn(bar_width=None),
        "[self.progress.percentage]{task.percentage:>3.1f}%")

    def __init__(self, presigned_url: str, source_file: str, total_size: int,
                 name: str):
        self.presigned_url = presigned_url
        self.source_file = source_file
        self.total_size = total_size
        self.name = name

    @staticmethod
    def upload_s3_data(url: str, data: bytes, retries: int,
                       disconnect_retry_wait_seconds: int) -> str:
        """
        Send data to s3 url

        Parameters
        ----------
        url: str
            S3 url string to send data to
        data: bytes
             Bytes of data to send to S3
        retries: int
            Amount of retries
        disconnect_retry_wait_seconds: int
            Amount of seconds between disconnect retry

        Returns
        -------
        str
            ETag from response
        """
        disconnect_retries = retries
        while disconnect_retries > 0:
            try:
                retries = Retry(total=10)
                with requests.Session() as s:
                    s.mount('https://', HTTPAdapter(max_retries=retries))
                    response = s.put(url, data=data)
                    if 'ETag' not in response.headers:
                        raise ValueError(
                            f"Unexpected response from S3, response: {response.content}"
                        )
                    return response.headers['ETag']
            except BrokenPipeError:
                time.sleep(disconnect_retry_wait_seconds)
                disconnect_retries -= 1

        raise ValueError("Unable to upload file after multiple attempts")

    def upload(self) -> None:
        """
        Upload files from source dir into target path in S3
        """
        task_id = self.progress.add_task("upload",
                                         filename=self.name,
                                         total=self.total_size)
        self.progress.start()
        try:
            with open(self.source_file, 'rb') as f:
                data = f.read()
            self.upload_s3_data(self.presigned_url, data, self.retries,
                                self.disconnect_retry_wait_seconds)
            self.progress.update(task_id, advance=len(data))
        finally:
            self.progress.stop()



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class SubscriptionCheckoutBody(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'product_id': 'str',
        'redirect_url_failed': 'str',
        'redirect_url_succeeded': 'str',
        'seats': 'int'
    }

    attribute_map = {
        'product_id': 'productId',
        'redirect_url_failed': 'redirectUrlFailed',
        'redirect_url_succeeded': 'redirectUrlSucceeded',
        'seats': 'seats'
    }

    def __init__(self, product_id: 'str' =None, redirect_url_failed: 'str' =None, redirect_url_succeeded: 'str' =None, seats: 'int' =None):  # noqa: E501
        """SubscriptionCheckoutBody - a model defined in Swagger"""  # noqa: E501
        self._product_id = None
        self._redirect_url_failed = None
        self._redirect_url_succeeded = None
        self._seats = None
        self.discriminator = None
        if product_id is not None:
            self.product_id = product_id
        if redirect_url_failed is not None:
            self.redirect_url_failed = redirect_url_failed
        if redirect_url_succeeded is not None:
            self.redirect_url_succeeded = redirect_url_succeeded
        if seats is not None:
            self.seats = seats

    @property
    def product_id(self) -> 'str':
        """Gets the product_id of this SubscriptionCheckoutBody.  # noqa: E501


        :return: The product_id of this SubscriptionCheckoutBody.  # noqa: E501
        :rtype: str
        """
        return self._product_id

    @product_id.setter
    def product_id(self, product_id: 'str'):
        """Sets the product_id of this SubscriptionCheckoutBody.


        :param product_id: The product_id of this SubscriptionCheckoutBody.  # noqa: E501
        :type: str
        """

        self._product_id = product_id

    @property
    def redirect_url_failed(self) -> 'str':
        """Gets the redirect_url_failed of this SubscriptionCheckoutBody.  # noqa: E501


        :return: The redirect_url_failed of this SubscriptionCheckoutBody.  # noqa: E501
        :rtype: str
        """
        return self._redirect_url_failed

    @redirect_url_failed.setter
    def redirect_url_failed(self, redirect_url_failed: 'str'):
        """Sets the redirect_url_failed of this SubscriptionCheckoutBody.


        :param redirect_url_failed: The redirect_url_failed of this SubscriptionCheckoutBody.  # noqa: E501
        :type: str
        """

        self._redirect_url_failed = redirect_url_failed

    @property
    def redirect_url_succeeded(self) -> 'str':
        """Gets the redirect_url_succeeded of this SubscriptionCheckoutBody.  # noqa: E501


        :return: The redirect_url_succeeded of this SubscriptionCheckoutBody.  # noqa: E501
        :rtype: str
        """
        return self._redirect_url_succeeded

    @redirect_url_succeeded.setter
    def redirect_url_succeeded(self, redirect_url_succeeded: 'str'):
        """Sets the redirect_url_succeeded of this SubscriptionCheckoutBody.


        :param redirect_url_succeeded: The redirect_url_succeeded of this SubscriptionCheckoutBody.  # noqa: E501
        :type: str
        """

        self._redirect_url_succeeded = redirect_url_succeeded

    @property
    def seats(self) -> 'int':
        """Gets the seats of this SubscriptionCheckoutBody.  # noqa: E501


        :return: The seats of this SubscriptionCheckoutBody.  # noqa: E501
        :rtype: int
        """
        return self._seats

    @seats.setter
    def seats(self, seats: 'int'):
        """Sets the seats of this SubscriptionCheckoutBody.


        :param seats: The seats of this SubscriptionCheckoutBody.  # noqa: E501
        :type: int
        """

        self._seats = seats

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SubscriptionCheckoutBody, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'SubscriptionCheckoutBody') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, SubscriptionCheckoutBody):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'SubscriptionCheckoutBody') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class ProjectsIdBody(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'display_name': 'str',
        'name': 'str',
        'preferred_cluster': 'str',
        'quotas': 'V1Quotas',
        'same_compute_on_resume': 'bool'
    }

    attribute_map = {
        'description': 'description',
        'display_name': 'displayName',
        'name': 'name',
        'preferred_cluster': 'preferredCluster',
        'quotas': 'quotas',
        'same_compute_on_resume': 'sameComputeOnResume'
    }

    def __init__(self, description: 'str' =None, display_name: 'str' =None, name: 'str' =None, preferred_cluster: 'str' =None, quotas: 'V1Quotas' =None, same_compute_on_resume: 'bool' =None):  # noqa: E501
        """ProjectsIdBody - a model defined in Swagger"""  # noqa: E501
        self._description = None
        self._display_name = None
        self._name = None
        self._preferred_cluster = None
        self._quotas = None
        self._same_compute_on_resume = None
        self.discriminator = None
        if description is not None:
            self.description = description
        if display_name is not None:
            self.display_name = display_name
        if name is not None:
            self.name = name
        if preferred_cluster is not None:
            self.preferred_cluster = preferred_cluster
        if quotas is not None:
            self.quotas = quotas
        if same_compute_on_resume is not None:
            self.same_compute_on_resume = same_compute_on_resume

    @property
    def description(self) -> 'str':
        """Gets the description of this ProjectsIdBody.  # noqa: E501


        :return: The description of this ProjectsIdBody.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description: 'str'):
        """Sets the description of this ProjectsIdBody.


        :param description: The description of this ProjectsIdBody.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def display_name(self) -> 'str':
        """Gets the display_name of this ProjectsIdBody.  # noqa: E501


        :return: The display_name of this ProjectsIdBody.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name: 'str'):
        """Sets the display_name of this ProjectsIdBody.


        :param display_name: The display_name of this ProjectsIdBody.  # noqa: E501
        :type: str
        """

        self._display_name = display_name

    @property
    def name(self) -> 'str':
        """Gets the name of this ProjectsIdBody.  # noqa: E501


        :return: The name of this ProjectsIdBody.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: 'str'):
        """Sets the name of this ProjectsIdBody.


        :param name: The name of this ProjectsIdBody.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def preferred_cluster(self) -> 'str':
        """Gets the preferred_cluster of this ProjectsIdBody.  # noqa: E501


        :return: The preferred_cluster of this ProjectsIdBody.  # noqa: E501
        :rtype: str
        """
        return self._preferred_cluster

    @preferred_cluster.setter
    def preferred_cluster(self, preferred_cluster: 'str'):
        """Sets the preferred_cluster of this ProjectsIdBody.


        :param preferred_cluster: The preferred_cluster of this ProjectsIdBody.  # noqa: E501
        :type: str
        """

        self._preferred_cluster = preferred_cluster

    @property
    def quotas(self) -> 'V1Quotas':
        """Gets the quotas of this ProjectsIdBody.  # noqa: E501


        :return: The quotas of this ProjectsIdBody.  # noqa: E501
        :rtype: V1Quotas
        """
        return self._quotas

    @quotas.setter
    def quotas(self, quotas: 'V1Quotas'):
        """Sets the quotas of this ProjectsIdBody.


        :param quotas: The quotas of this ProjectsIdBody.  # noqa: E501
        :type: V1Quotas
        """

        self._quotas = quotas

    @property
    def same_compute_on_resume(self) -> 'bool':
        """Gets the same_compute_on_resume of this ProjectsIdBody.  # noqa: E501


        :return: The same_compute_on_resume of this ProjectsIdBody.  # noqa: E501
        :rtype: bool
        """
        return self._same_compute_on_resume

    @same_compute_on_resume.setter
    def same_compute_on_resume(self, same_compute_on_resume: 'bool'):
        """Sets the same_compute_on_resume of this ProjectsIdBody.


        :param same_compute_on_resume: The same_compute_on_resume of this ProjectsIdBody.  # noqa: E501
        :type: bool
        """

        self._same_compute_on_resume = same_compute_on_resume

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ProjectsIdBody, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'ProjectsIdBody') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, ProjectsIdBody):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ProjectsIdBody') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

import pprint
import re  # noqa: F401

from typing import TYPE_CHECKING

import six

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class CloudspaceIdMetricBody(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'completed_at': 'datetime',
        'instance_type': 'str',
        'started_at': 'datetime',
        'step': 'str'
    }

    attribute_map = {
        'completed_at': 'completedAt',
        'instance_type': 'instanceType',
        'started_at': 'startedAt',
        'step': 'step'
    }

    def __init__(self, completed_at: 'datetime' =None, instance_type: 'str' =None, started_at: 'datetime' =None, step: 'str' =None):  # noqa: E501
        """CloudspaceIdMetricBody - a model defined in Swagger"""  # noqa: E501
        self._completed_at = None
        self._instance_type = None
        self._started_at = None
        self._step = None
        self.discriminator = None
        if completed_at is not None:
            self.completed_at = completed_at
        if instance_type is not None:
            self.instance_type = instance_type
        if started_at is not None:
            self.started_at = started_at
        if step is not None:
            self.step = step

    @property
    def completed_at(self) -> 'datetime':
        """Gets the completed_at of this CloudspaceIdMetricBody.  # noqa: E501


        :return: The completed_at of this CloudspaceIdMetricBody.  # noqa: E501
        :rtype: datetime
        """
        return self._completed_at

    @completed_at.setter
    def completed_at(self, completed_at: 'datetime'):
        """Sets the completed_at of this CloudspaceIdMetricBody.


        :param completed_at: The completed_at of this CloudspaceIdMetricBody.  # noqa: E501
        :type: datetime
        """

        self._completed_at = completed_at

    @property
    def instance_type(self) -> 'str':
        """Gets the instance_type of this CloudspaceIdMetricBody.  # noqa: E501


        :return: The instance_type of this CloudspaceIdMetricBody.  # noqa: E501
        :rtype: str
        """
        return self._instance_type

    @instance_type.setter
    def instance_type(self, instance_type: 'str'):
        """Sets the instance_type of this CloudspaceIdMetricBody.


        :param instance_type: The instance_type of this CloudspaceIdMetricBody.  # noqa: E501
        :type: str
        """

        self._instance_type = instance_type

    @property
    def started_at(self) -> 'datetime':
        """Gets the started_at of this CloudspaceIdMetricBody.  # noqa: E501


        :return: The started_at of this CloudspaceIdMetricBody.  # noqa: E501
        :rtype: datetime
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at: 'datetime'):
        """Sets the started_at of this CloudspaceIdMetricBody.


        :param started_at: The started_at of this CloudspaceIdMetricBody.  # noqa: E501
        :type: datetime
        """

        self._started_at = started_at

    @property
    def step(self) -> 'str':
        """Gets the step of this CloudspaceIdMetricBody.  # noqa: E501


        :return: The step of this CloudspaceIdMetricBody.  # noqa: E501
        :rtype: str
        """
        return self._step

    @step.setter
    def step(self, step: 'str'):
        """Sets the step of this CloudspaceIdMetricBody.


        :param step: The step of this CloudspaceIdMetricBody.  # noqa: E501
        :type: str
        """

        self._step = step

    def to_dict(self) -> dict:
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CloudspaceIdMetricBody, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self) -> str:
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self) -> str:
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other: 'CloudspaceIdMetricBody') -> bool:
        """Returns true if both objects are equal"""
        if not isinstance(other, CloudspaceIdMetricBody):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'CloudspaceIdMetricBody') -> bool:
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    external/v1/auth_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    NOTE
    ----
    standard swagger-codegen-cli for this python client has been modified
    by custom templates. The purpose of these templates is to include
    typing information in the API and Model code. Please refer to the
    main grid repository for more info
"""

from __future__ import absolute_import

import re  # noqa: F401
from typing import TYPE_CHECKING, Any

# python 2 and python 3 compatibility library
import six

from lightning_cloud.openapi.api_client import ApiClient

if TYPE_CHECKING:
    from datetime import datetime
    from lightning_cloud.openapi.models import *

class DatasetServiceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def dataset_service_create_dataset(self, body: 'ProjectIdDatasetsBody', project_id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_create_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_create_dataset(body, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectIdDatasetsBody body: (required)
        :param str project_id: (required)
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.dataset_service_create_dataset_with_http_info(body, project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.dataset_service_create_dataset_with_http_info(body, project_id, **kwargs)  # noqa: E501
            return data

    def dataset_service_create_dataset_with_http_info(self, body: 'ProjectIdDatasetsBody', project_id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_create_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_create_dataset_with_http_info(body, project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProjectIdDatasetsBody body: (required)
        :param str project_id: (required)
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method dataset_service_create_dataset" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `dataset_service_create_dataset`")  # noqa: E501
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `dataset_service_create_dataset`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['projectId'] = params['project_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/projects/{projectId}/datasets', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1Dataset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def dataset_service_delete_dataset(self, project_id: 'str', id: 'str', **kwargs) -> 'V1DeleteDatasetResponse':  # noqa: E501
        """dataset_service_delete_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_delete_dataset(project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str id: (required)
        :return: V1DeleteDatasetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.dataset_service_delete_dataset_with_http_info(project_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.dataset_service_delete_dataset_with_http_info(project_id, id, **kwargs)  # noqa: E501
            return data

    def dataset_service_delete_dataset_with_http_info(self, project_id: 'str', id: 'str', **kwargs) -> 'V1DeleteDatasetResponse':  # noqa: E501
        """dataset_service_delete_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_delete_dataset_with_http_info(project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str id: (required)
        :return: V1DeleteDatasetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_id', 'id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method dataset_service_delete_dataset" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `dataset_service_delete_dataset`")  # noqa: E501
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `dataset_service_delete_dataset`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['projectId'] = params['project_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/projects/{projectId}/datasets/{id}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1DeleteDatasetResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def dataset_service_get_dataset(self, project_id: 'str', id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_get_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_get_dataset(project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str id: (required)
        :param str cluster_id:
        :param str creator_id:
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.dataset_service_get_dataset_with_http_info(project_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.dataset_service_get_dataset_with_http_info(project_id, id, **kwargs)  # noqa: E501
            return data

    def dataset_service_get_dataset_with_http_info(self, project_id: 'str', id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_get_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_get_dataset_with_http_info(project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str id: (required)
        :param str cluster_id:
        :param str creator_id:
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_id', 'id', 'cluster_id', 'creator_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method dataset_service_get_dataset" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `dataset_service_get_dataset`")  # noqa: E501
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `dataset_service_get_dataset`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['projectId'] = params['project_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'cluster_id' in params:
            query_params.append(('clusterId', params['cluster_id']))  # noqa: E501
        if 'creator_id' in params:
            query_params.append(('creatorId', params['creator_id']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/projects/{projectId}/datasets/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1Dataset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def dataset_service_list_datasets(self, project_id: 'str', **kwargs) -> 'V1ListDatasetsResponse':  # noqa: E501
        """dataset_service_list_datasets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_list_datasets(project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str cluster_id:
        :param str creator_id:
        :return: V1ListDatasetsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.dataset_service_list_datasets_with_http_info(project_id, **kwargs)  # noqa: E501
        else:
            (data) = self.dataset_service_list_datasets_with_http_info(project_id, **kwargs)  # noqa: E501
            return data

    def dataset_service_list_datasets_with_http_info(self, project_id: 'str', **kwargs) -> 'V1ListDatasetsResponse':  # noqa: E501
        """dataset_service_list_datasets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_list_datasets_with_http_info(project_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str project_id: (required)
        :param str cluster_id:
        :param str creator_id:
        :return: V1ListDatasetsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['project_id', 'cluster_id', 'creator_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method dataset_service_list_datasets" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `dataset_service_list_datasets`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['projectId'] = params['project_id']  # noqa: E501

        query_params = []
        if 'cluster_id' in params:
            query_params.append(('clusterId', params['cluster_id']))  # noqa: E501
        if 'creator_id' in params:
            query_params.append(('creatorId', params['creator_id']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/projects/{projectId}/datasets', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1ListDatasetsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def dataset_service_update_dataset(self, body: 'DatasetsIdBody', project_id: 'str', id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_update_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_update_dataset(body, project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param DatasetsIdBody body: (required)
        :param str project_id: (required)
        :param str id: (required)
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.dataset_service_update_dataset_with_http_info(body, project_id, id, **kwargs)  # noqa: E501
        else:
            (data) = self.dataset_service_update_dataset_with_http_info(body, project_id, id, **kwargs)  # noqa: E501
            return data

    def dataset_service_update_dataset_with_http_info(self, body: 'DatasetsIdBody', project_id: 'str', id: 'str', **kwargs) -> 'V1Dataset':  # noqa: E501
        """dataset_service_update_dataset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.dataset_service_update_dataset_with_http_info(body, project_id, id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param DatasetsIdBody body: (required)
        :param str project_id: (required)
        :param str id: (required)
        :return: V1Dataset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'project_id', 'id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method dataset_service_update_dataset" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `dataset_service_update_dataset`")  # noqa: E501
        # verify the required parameter 'project_id' is set
        if ('project_id' not in params or
                params['project_id'] is None):
            raise ValueError("Missing the required parameter `project_id` when calling `dataset_service_update_dataset`")  # noqa: E501
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `dataset_service_update_dataset`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'project_id' in params:
            path_params['projectId'] = params['project_id']  # noqa: E501
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/v1/projects/{projectId}/datasets/{id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='V1Dataset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
